#include <stdlib.h>
#include <math.h>
#include <mex.h>
#include <vector>
#define bin_num 9

using namespace std;   

void HoG(double *pixels, double *params, int *img_size, double *dth_des, unsigned int grayscale){
    
    const float pi = 3.1415926536;
    
    double frame_size     =  params[0];
    int block_size    = (int) params[1];
    int orient        = (int) params[2];
    
    
    
    unsigned int img_width  = img_size[1];
    unsigned int img_height = img_size[0];
    
    float dx, dy, grad_or, grad_mag, temp_mag;
    float Xc, Yc, Oc, block_norm;
    
    double bin[bin_num];  
    vector<vector<vector<double> > > block(block_size, vector<vector<double> > (block_size, vector<double> (bin_num, 0.0) ) );  
    
    //Calculate gradients

    for (unsigned float i=0;i<bin_num;i++) 
    { 
          bin[i]=0;
      for(unsigned int y=0; y<img_height; y++) 
      {
        for(unsigned int x=0; x<img_width; x++) 
        {
              if(x==0) dx = pixels[y +(x+1)*img_height];
                else
                {
                    if (x==img_width-1) dx = -pixels[y + (x-1)*img_height];
                    else dx = pixels[y+(x+1)*img_height] - pixels[y + (x-1)*img_height];
                }
                if(y==0) dy = -pixels[y+1+x*img_height];
                else
                {
                    if (y==img_height-1) dy = pixels[y-1+x*img_height];
                    else dy = -pixels[y+1+x*img_height] + pixels[y-1+x*img_height];
                }
            
            grad_mag= sqrt(dx*dx + dy*dy);
            grad_or= atan2(dy, dx);
        //    dth_des[y+x*10]=grad_mag;
            if ((grad_or>=(-pi+i*2*pi/bin_num))&&(grad_or <(-pi+(i+1)*2*pi/bin_num)))
            {
              bin[i]=bin[i]+grad_mag;  
            }
        }
    }
         
 }

 
    
    //Block normalization
    
   
            block_norm=0;

            for(unsigned int k=0; k<bin_num; k++)
           {
              block_norm+=bin[k]*bin[k];
           }
            block_norm=sqrt(block_norm)+0.01;
        for (int i=0;i<bin_num;i++)
   {
          dth_des[i]=bin[i]/block_norm;
    }

}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    
    double *pixels, *dth_des, *params;
    int  block_size;
    int img_size[2];
    unsigned int grayscale = 1;
    
    if (nlhs>1)  mexErrMsgTxt("Too many output arguments");
    if (nrhs==0) mexErrMsgTxt("No Image -> No HoG");
    
    if (mxGetClassID(prhs[0])!=6) mexErrMsgTxt("Matrix is not of type double");
    
    pixels     = mxGetPr(prhs[0]);    
    
    img_size[0] = mxGetM(prhs[0]);
    img_size[1]  = mxGetN(prhs[0]);
    if (mxGetNumberOfDimensions(prhs[0])==3){
        img_size[1] /= 3;
        grayscale = 0;
    }
    
    if (nrhs>1){
        params     = mxGetPr(prhs[1]);
        if (params[0]<=0) mexErrMsgTxt("Cell size must be positive");
        if (params[1]<=0) mexErrMsgTxt("Block size must be positive");
    }
    else {
        params = new double[5];
        params[0]=8;
        params[1]=2;
        params[2]=0;
    }
      
    block_size    = (int) params[1];     


    plhs[0] = mxCreateDoubleMatrix(9,1, mxREAL);
    dth_des = mxGetPr(plhs[0]);
    
    HoG(pixels, params, img_size, dth_des, grayscale);
    if (nrhs==1) delete[] params;
}
